<?php

namespace FiloBlu\CegidY2\Tools;

use DOMDocument;
use DOMElement;
use DOMXPath;

/**
 * Class SystemXmlGenerator
 * @package FiloBlu\CegidY2\Tools
 */
class SystemXmlGenerator
{

    /**
     * @var array
     */
    protected $endpoints;

    /**
     * SystemXmlGenerator constructor.
     * @param $endpoints
     */
    public function __construct($endpoints)
    {
        $this->endpoints = $endpoints;
    }

    /**
     *
     */
    public function generate()
    {
        $systemXML = new DOMDocument('1.0', 'UTF-8');
        $config = $systemXML->createElement('config');
        $config->setAttribute('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance');
        $config->setAttribute('xsi:noNamespaceSchemaLocation', 'urn:magento:module:Magento_Config:etc/system_file.xsd');

        $system = $systemXML->createElement('system');
        $globalTab = $systemXML->createElement('tab');

        $globalTab->setAttribute('id', 'filoblu_tab');
        $globalTab->setAttribute('translate', 'label');
        $globalTab->setAttribute('sortOrder', '2000');

        $globalLabel = $systemXML->createElement('label');
        $globalLabel->textContent = 'FiloBlu';

        $globalTab->appendChild($globalLabel);
        $system->appendChild($globalTab);

        $section = $systemXML->createElement('section');
        $section->setAttribute('id', 'filoblu_cegid_y2');
        $section->setAttribute('translate', 'label');
        $section->setAttribute('sortOrder', '10');
        $section->setAttribute('showInDefault', '1');
        $section->setAttribute('showInWebsite', '1');
        $section->setAttribute('showInStore', '1');

        $system->appendChild($section);

        $tagClass = $systemXML->createElement('class');
        $label = $systemXML->createElement('label');
        $resource = $systemXML->createElement('resource');
        $tab = $systemXML->createElement('tab');
        $tab->textContent = 'filoblu_tab';
        $section->appendChild($tab);

        $tagClass->textContent = 'separator-top';
        $label->textContent = 'Cegid Endpoint Configuration';
        $resource->textContent = 'FiloBlu_CegidY2::endpoint_configuration';

        $section->appendChild($tagClass);
        $section->appendChild($label);
        $section->appendChild($resource);

        $section->appendChild($this->generateGroup($systemXML, 'default', 'Default configuration', false));

        foreach ($this->endpoints as $service) {
            $section->appendChild($this->generateGroup($systemXML, $service, "Service {$service} configuration"));
        }

        $config->appendChild($system);

        $systemXML->appendChild($config);
        $systemXML->preserveWhiteSpace = false;
        $systemXML->formatOutput = true;
        file_put_contents(__DIR__ . '/../etc/adminhtml/system.xml', $systemXML->saveXML());

        $this->updateDiXml();
    }

    /**
     * @param DOMDocument $xml
     * @param string $service
     * @param $labelString
     * @param bool $withHost
     * @return DOMElement
     */
    function generateGroup(DOMDocument $xml, $service, $labelString, $withHost = true)
    {
        $group = $xml->createElement('group');
        $group->setAttribute('id', Utils::getServiceId($service));
        $group->setAttribute('translate', 'label');
        $group->setAttribute('type', 'text');
        $group->setAttribute('sortOrder', '150');
        $group->setAttribute('showInDefault', '1');
        $group->setAttribute('showInWebsite', '1');
        $group->setAttribute('showInStore', '1');

        $label = $xml->createElement('label');
        $label->textContent = $labelString;
        $group->appendChild($label);

        if ($withHost) {
            $host = $this->generateField($xml, 'url', 'Url', 'text', "https://www.example.com/Y2/{$service}WcfService.svc?singlewsdl");
            $group->appendChild($host);
        }

        $login = $this->generateField($xml, 'login', 'Login', 'text');
        $password = $this->generateField($xml, 'password', 'Password', 'text');
        $database = $this->generateField($xml, 'database', 'Database', 'text');

        $group->appendChild($login);
        $group->appendChild($password);
        $group->appendChild($database);

        return $group;
    }

    /**
     * @param DOMDocument $xml
     * @param $id
     * @param $labelString
     * @param $type
     * @param null $commentString
     * @return DOMElement
     */
    public function generateField(DOMDocument $xml, $id, $labelString, $type, $commentString = null)
    {
        $field = $xml->createElement('field');

        $field->setAttribute('id', $id);
        $field->setAttribute('type', $type);

        $label = $xml->createElement('label');
        $label->textContent = $labelString;
        $field->appendChild($label);
        $field->setAttribute('showInDefault', '1');
        $field->setAttribute('showInWebsite', '1');
        $field->setAttribute('showInStore', '1');
        $field->setAttribute('sortOrder', '1');

        if ($commentString !== null) {
            $comment = $xml->createElement('comment');
            $comment->textContent = $commentString;
            $field->appendChild($comment);
        }

        return $field;
    }

    /**
     *
     */
    public function updateDiXml()
    {
        $xml = new DOMDocument('1.0', 'UTF-8');

        $xml->preserveWhiteSpace = false;
        $xml->formatOutput = true;
        $xml->load(__DIR__ . '/../etc/di.xml');
        $xpath = new DOMXPath($xml);
        $config = $xpath->query('/config')->item(0);
        $factories = $xpath->query('//virtualType[@type="FiloBlu\CegidY2\Service\ServiceProvider"]');

        $serviceMap =
            $xpath->
            query('//type[@name="FiloBlu\CegidY2\Service\CredentialProvider"]/arguments/argument[@name="serviceMap"]')
                ->item(0);

        if ($serviceMap) {
            while ($element = $serviceMap->childNodes->item(0)) {
                $serviceMap->removeChild($element);
            }
        }
        /*else{
            $type = $xml->createElement('type');
            $type->setAttribute('name', 'FiloBlu\\CegidY2\\Service\\CredentialProvider');
            $arguments  = $xml->createElement('arguments');
            $type->appendChild($arguments);
            $serviceMap = $xml->createElement('argument');
            $serviceMap->setAttribute('name', 'serviceMap');
            $serviceMap->setAttribute('xsi:type', 'array');
            $arguments->appendChild($serviceMap);
            $config->appendChild($type);
        }*/

        foreach ($this->endpoints as $service) {
            $item = $xml->createElement('item');
            $item->setAttribute('name', $service);
            $item->setAttribute('xsi:type', 'string');
            $item->textContent = Utils::getServiceId($service);
            $serviceMap->appendChild($item);
        }

        $total = $factories->length;
        for ($i = 0; $i < $total; $i++) {
            $config->removeChild($factories->item($i));
        }

        $items = glob(__DIR__ . '/../Api/*/ServiceType/*.php');

        foreach ($items as $i) {
            $part = explode('/', $i);
            $file = array_pop($part);
            array_pop($part);
            $service = array_pop($part);
            $serviceType = pathinfo($file, PATHINFO_FILENAME);
            $config->appendChild($this->createFactory($xml, $service, $serviceType));
        }

        $xml->save(__DIR__ . '/../etc/di.xml');
    }

    /**
     * @param DOMDocument $xml
     * @param string $service
     * @param string $serviceType
     * @return DOMElement
     */

    public function createFactory(DOMDocument $xml, $service, $serviceType)
    {
        $name = "FiloBlu\\CegidY2\\Api\\{$service}\\ServiceType\\{$serviceType}";
        $factory = $xml->createElement('virtualType');
        $factory->setAttribute('name', "{$name}Factory");
        $factory->setAttribute('type', 'FiloBlu\\CegidY2\\Service\\ServiceProvider');

        $arguments = $xml->createElement('arguments');

        $instanceName = $xml->createElement('argument');
        $instanceName->setAttribute('name', 'instanceName');
        $instanceName->setAttribute('xsi:type', 'string');
        $instanceName->textContent = $name;

        $serviceId = $xml->createElement('argument');
        $serviceId->setAttribute('name', 'serviceId');
        $serviceId->setAttribute('xsi:type', 'string');
        $serviceId->textContent = Utils::getServiceId($service);

        $wsdlClassMap = $xml->createElement('argument');
        $wsdlClassMap->setAttribute('name', 'wsdlClassMap');
        $wsdlClassMap->setAttribute('xsi:type', 'string');
        $wsdlClassMap->textContent = "FiloBlu\\CegidY2\\Api\\{$service}\\ClassMap";

        $arguments->appendChild($instanceName);
        $arguments->appendChild($wsdlClassMap);
        $arguments->appendChild($serviceId);
        $factory->appendChild($arguments);
        return $factory;
    }
}





